/*
 * $Id: $
 * (C) 2000 - 2010 CARNOT AG
 */
package org.eclipse.stardust.clustering;

import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextClosedEvent;

import org.eclipse.stardust.common.Action;
import org.eclipse.stardust.common.log.LogManager;
import org.eclipse.stardust.common.log.Logger;

/**
 * Enhancement of the standard {@link DaemonWatchDog} class to be used in a
 * container-managed environment to cleanly shutdown the daemons.<br/><br/>
 * IMPORTANT: In order for this to work the vital resources (like AuditTrail,
 * JMS queues, connection factories, etc.) must be managed by the container outside
 * of the IPP application context. This class will fail with exceptions in a standard
 * RAD environment on Tomcat, because the connection pool and other resources might
 * closed before the shutdown mechanism is executed! 
 * 
 * @author JanHendrik.Scheufen
 */
public class ManagedDaemonWatchDog extends DaemonWatchDog implements ApplicationListener
{
	private static Logger logger = LogManager.getLogger(ManagedDaemonWatchDog.class);

	@SuppressWarnings("unchecked")
	public void onApplicationEvent(ApplicationEvent event)
	{
		if (event instanceof ContextClosedEvent) {
			forkingService.isolate(new Action() {
				public Object execute() {
					logger.info("Shutting down IPP daemons ...");
					executeForAllDaemons(new StopDaemonAction());
					return null;
				}
			});
		}
	}
	
}
